@@ -3,7 +3,7 @@ package Parse::RecDescent::FAQ;
 use vars qw($VERSION);
 
 
-our $VERSION = sprintf '%s', q$Revision: 3.19 $ =~ /Revision:\s+(\S+)\s+/ ;
+our $VERSION = sprintf '%s', q$Revision: 3.2 $ =~ /Revision:\s+(\S+)\s+/ ;
 
 1;
 __END__
@@ -36,13 +36,13 @@ or not.
 
 
 
+
 =back
 
 Also see the "Left-recursion" section under "PARSER BEHAVIOR"
 
 =head1 DEBUGGING
 
-=head2 Flags you can set
 Paraphrased from Yves Orton:
 
 To understand more about why your parser is not behaving as it should
@@ -53,26 +53,6 @@ take advantage of the follow variables:
    $::RD_HINT         # if defined, also suggestion remedies
    $::RD_TRACE        # if defined, also trace parsers' behaviour
 
-=head2 Making warning line numbers correspond to your grammar
-
-How do I match the line numbers with the actual contents of my
-script?
-
-At present, you can't (but that's on the ToDo list).
-Setting C<$::RD_TRACE> can be useful though:
-
-Once you've run with C<$RD_TRACE>, do this:
-
-        perl -w RD_TRACE
-
-Then go and examine the actual line numbers given for the error
-in the file C<RD_TRACE>.
-
-That will show you the actual generated code that's the problem.
-
-That code will, in turn, give you a hint where the problem is in the
-grammar (e.g. find out which subroutine it's in, which will tell you the
-name of the offending rule).
 
 
 =head1 PARSER BEHAVIOR
@@ -358,6 +338,26 @@ as well.
 =back
 
 
+=head2 Making warning line numbers correspond to your grammar
+
+How do I match the line numbers with the actual contents of my
+script?
+
+At present, you can't (but that's on the ToDo list).
+Setting C<$::RD_TRACE> can be useful though:
+
+Once you've run with C<$RD_TRACE>, do this:
+
+        perl -w RD_TRACE
+
+Then go and examine the actual line numbers given for the error
+in the file C<RD_TRACE>.
+
+That will show you the actual generated code that's the problem.
+
+That code will, in turn, give you a hint where the problem is in the
+grammar (e.g. find out which subroutine it's in, which will tell you the
+name of the offending rule).
 
 =head1 IGNORABLE TOKENS 
 
@@ -382,7 +382,7 @@ Do something like this:
 
 =head1 NEWLINE PROCESSING
 
-=head2 Parsing Windows Init (.ini) Files
+=head2 Parsing Windows Init Files
 
 I'm trying ot use Parse::RecDescent to parse a configuration file which
 looks like this:
@@ -424,10 +424,6 @@ looks like this:
 Also take a look at the example in section 11.2 in "Data Munging with Perl"
 by Dave Cross.
 
-=item * And the newest answer by the tireless Randal L. Schwartz: 
-
-  L<http://www.stonehenge.com/merlyn/UnixReview/col40.html>
-
 =back
 
 
@@ -570,9 +566,6 @@ So if you would like to state the test in the positive, then do this:
 
 =head1 MODULAR / GENERATIVE / CREATIVE / HAIRY PARSING
 
-Be sure to see the example see the RESOURCES section at the end
-for even more nosebleed parsing examples.
-
 =head2 Macro processing
 
 In a RecDescent grammar, I'd like to add support in my language for
@@ -912,18 +905,6 @@ Then just define C<QuotedText>, C<Comma>, and you're done!
 
 =head1 OPTIMIZING YOUR GRAMMARS
 
-=head2 Dan Sugalski's Practice and Theory of Optimizing PRD Grammars
-
-This link is excellent. So excellent in fact, that I took the time to mirror
-it to my site to avoid it ever being lost:
-
-L<http://www.urth.org/~metaperl/domains/semantic-elements.com/perl/prd/dans-blog/000235.html>
-
-This link provides an excellent discussion of hits and misses in optimizing a
-grammar whose initial parse time started out at around minutes. After whittling
-it down to 5 minutes, Dan is still not happy and continues with even
-more aggressive optimizations! Well worth the read.
-
 =head2 Eliminate backtracking when possible
 
 Let's take a look at two computationally equivalent grammars:
@@ -1723,10 +1704,59 @@ to this:
 
 =back
 
+=head1 REGULAR EXPRESSIONS
+
+=head2 Shortest match instead of longest match
+
+What is the Perl idiom for getting the leftmost shortest match?
+For instance, so:
+
+ $a = "banana";
+ $a =~ /b.*n/;  # but different
+ print $&;
+
+yields C<banan>. How can I get it to yield
+C<ban>?
+
+=over 4
+
+=item * Just use the non-greedy quantifier modifier, C<?> :
+
+    $a =~ /b.*?n/;
+
+=back
+
+=head2 Tutorial on positive and negative lookahead and lookbehind
+regexps:
+
+ http://lists.isb.sdnpk.org/pipermail/comp-list/2002-August/001156.html
 
 
-=head1 PROGRAMMING TOPICS GERMANE TO Parse::RecDescent USE
+=head2  Text::Balanced  and Regexp::Common::balanced
 
+Use this instead of writing hairy regular expressions to match certain
+common "balanced" forms of text, such as tags and parenthesized text.
+
+See http://perlmonks.org/index.pl?node_id=208285
+for an example of where it would be better suited for the task than
+Parse::RecDescent.
+
+=head2 Printed Resources
+
+=head3 Japhy's Book on Regular Expressions
+
+Ok, so he hasn't named it yet. It's available online for free!
+
+  http://japhy.perlmonk.org/book
+
+=head3 "Mastering Regular Expressions" by Jeffrey Freidl
+
+
+You still need to know when to use C</.*/> or C</.+/> or C</[^x]*/>
+
+
+
+=head1 Programming Topics Germane to Parse::RecDescent Use
 
 =head2 Double vs Single-quoted strings
 
@@ -1857,17 +1887,13 @@ a production the same regardless of whether P::RD returns a string or a
 list of string.
 
 =over 4
-=item * Use L<Scalar::Listify|Scalar::Listify> from CPAN.
+=item * Use Scalar::Listify from CPAN.
 
 =back
 
-=head2 Shallow versus Deep Copying 
+=head2 A tutorial on Shallow versus Deep Copying 
 
-=head3 An article by Randal Schwartz
-    
-    http://www.stonehenge.com/merlyn/UnixReview/col30.html
-
-=head3 A tutorial written by Philip Newton <nospam.newton@gmx.li>
+Written by "Philip 'Yes, that's my address' Newton" <nospam.newton@gmx.li>
 
 Start off with an array of (references to) arrays: 
 
@@ -2170,157 +2196,93 @@ is by far the likeliest explanation.
 
 =back
 
-=head2 Data::MultiValuedHash on search.cpan.org
-
-L<Data::MultiValuedHash> allows for
-transparent manipulation of single or multiply-valued Perl hash values.
-If you've done any recdescent-grokking, you know how useful this can be
-when extracting information from a parse.
-
-=head2 Regular Expressions
-
-=head3 Shortest match instead of longest match
-
-What is the Perl idiom for getting the leftmost shortest match?
-For instance, so:
-
- $a = "banana";
- $a =~ /b.*n/;  # but different
- print $&;
-
-yields C<banan>. How can I get it to yield
-C<ban>?
-
-=over 4
-
-=item * Just use the non-greedy quantifier modifier, C<?> :
-
-    $a =~ /b.*?n/;
-
-=back
-
-=head3 Tutorial on positive and negative lookahead and lookbehind
-regexps:
-
- http://lists.isb.sdnpk.org/pipermail/comp-list/2002-August/001156.html
-
-
-=head3  Text::Balanced  and Regexp::Common::balanced
-
-Use this instead of writing hairy regular expressions to match certain
-common "balanced" forms of text, such as tags and parenthesized text.
-
-See http://perlmonks.org/index.pl?node_id=208285
-for an example of where it would be better suited for the task than
-Parse::RecDescent.
-
-=head3 Japhy's Book on Regular Expressions
-
-Ok, so he hasn't named it yet. It's available online for free!
-
-  http://japhy.perlmonk.org/book
-
-=head3 "Mastering Regular Expressions" by Jeffrey Freidl
-
-
-You still need to know when to use C</.*/> or C</.+/> or C</[^x]*/>
 
 
 =head1 RESOURCES
 
-=head2 Practical Parser Examples
+=over 4
 
-=head3 XSH - The XML Editing Shell
-
-XSH:
-
-L<http://xsh.sourceforge.net/>
-
-uses Parse::RecDescent
-
-=head3 Parsing Symbolic Expressions / Boolean Logic
-
-Says Stefan Mueller:
-
-Parse::RecDescent can easily parse such expressions. 
-You can find a working example grammar for more involved 
-*arithmetic* expressions in the L<Math::Symbolic::Parser> module that is 
-part of the L<Math::Symbolic> distribution. 
-It should be fairly easy to modify it to work for boolean logic. 
-
-=head3 MySql to Oracle schema conversion utility
+=item * MySql to Oracle schema conversion utility
 
 Written in Parse::RecDescent by Tim Bunce:
 
-L<http://groups.google.com/groups?q=recdescent&start=40&hl=en&scoring=d&rnum=41&selm=9km7b1%246vc%241%40FreeBSD.csie.NCTU.edu.tw>
-
-=head2 Web Resources
+ http://groups.google.com/groups?q=recdescent&start=40&hl=en&scoring=d&rnum=41&selm=9km7b1%246vc%241%40FreeBSD.csie.NCTU.edu.tw
 
 
-=head3 Hugh Myer's tips on Parse::RecDescent
+=item * Data::MultiValuedHash on search.cpan.org
 
-L<http://www.perlmonks.org/index.pl?node_id=180778>
+Transparent manipulation of single or multiply-valued Perl hash values.
 
-17 tips you cannot do without.
+=item * Parse::Recdescent tutorial at www.perl.com
 
-=head3 Craig's Parse::Recdescent Area
+http://www.perl.com/pub/a/2001/06/13/recdecent.html
 
-L<http://www.informatik.uni-trier.de/~smith/perl/recdescent/>
-    
-Contains an article (in English) published during the Zweiter Perl Workshop
-as well as some slides for a presentation:
+=item * "Safe undumping"
 
-
-=head3 http://www.PerlMonks.org and irc://efnet/perlhelp
-
-Useful sites to get fast help on Perl.
-
-
-=head2 Publications
-
-=head3 "Parse::Recdescent Tutorial", www.perl.com, 6/13/2001, Jeffrey Goff
-
-L<http://www.perl.com/pub/a/2001/06/13/recdecent.html>
-
-=head3 "Parsing Interesting Things", UnixReview 12/2001, Randal Schwartz
-
-L<http://www.stonehenge.com/merlyn/UnixReview/col40.html>
-
-=head3 "Safe Undumping", Linux Magazine, 10/2001, Randal Schwartz
-
-Randal Schwartz uses
+In this Linux Magazine article by Randal Schwartz uses
 Parse::RecDescent to parse Data::Dumper output. Not fast, but quite
 complete. 
 
-L<http://www.stonehenge.com/merlyn/LinuxMag/col29.html>
-
-=head3 "Data Munging with Perl" by Dave Cross
-
-Chapter 11 is focused on parsing with special emphasis on practical use of
-Parse::RecDescent.
-
-=head3 "Object-Oriented Perl" by Damian Conway
-
-This book will aid you in complexity management for large grammars.
+=item "Parsing Interesting Things", SysAdminMag.COM, December 2001, Randal Schwartz
 
 
-=head2 Non Parse::RecDescent Perl Parsers
 
-=head3 py
+=item * py
 
 py, which I assume is short for C<parse yacc>, is a program by
 Mark-Jason Dominus which parses GNU Bison output to produce Perl parsers.
 
 It is obtainable from http://perl.plover.com/py/
 
-=head3 Parse::YAPP
+=item * Parse::YAPP
 
 A bottom-up parser which will be familiar to those who
 have used Lex and Yacc. Parse::RecDescent is a top-down parser.
 
+=item * "Data Munging with Perl" by Dave Cross
 
+Chapter 11 is focused on parsing with special emphasis on practical use of
+Parse::RecDescent.
 
-=head1 SOURCES for FAQ MATERIAL
+=item * "Object-Oriented Perl" by Damian Conway
+
+This book will aid you in complexity management for large grammars.
+
+=item * http://www.PerlMonks.org
+
+A useful site to get fast help on Perl.
+
+=back
+
+=head1 AUTHOR
+
+The author of Parse::RecDescent::FAQ is Terrence Brannon <tbone@cpan.org>. 
+
+The author of Parse::RecDescent is Damian Conway. 
+
+The (unwitting) contributors to this FAQ
+
+=over 4 
+
+=item * Me
+
+=item * Damian Conway
+
+=item * Marcel Grunaer
+
+=item * Brent Dax
+
+=item * Randal L. Schwartz (merlyn), Perl hacker
+
+=item * lhoward of Perlmonks
+
+=item * Matthew Wickline
+
+=item * Gwynn Judd
+
+=back 
+
+=head1 SOURCE for FAQ MATERIAL
 
 I try to regularly scan all sources of Perl question-answer for FAQ material.
 If you can think of another source that is not listed below, I would
@@ -2344,35 +2306,4 @@ Conveniently available via NNTP at:
 
 =back
 
-=head1 AUTHOR
-
-The author of Parse::RecDescent::FAQ is Terrence Brannon <tbone@cpan.org>. 
-
-The author of Parse::RecDescent is Damian Conway. 
-
-The (unwitting) contributors to this FAQ
-
-=over 4 
-
-=item * Brent Dax
-
-=item * Damian Conway
-
-=item * Gwynn Judd
-
-=item * lhoward of Perlmonks
-
-=item * Marcel Grunaer
-
-=item * Matthew Wickline
-
-=item * Randal L. Schwartz
-
-=item * Stefan Mueller
-
-=item * Yves Orton
-
-=back 
-
-
 =cut
